@@ -36,18 +36,10 @@
 #define SQWRITE gp_port_usb_msg_write
 #define SQREAD  gp_port_usb_msg_read
 
-static unsigned char *
-digi_read_data (GPPort *port, unsigned char *data, int size)
-{
-	SQWRITE (port, 0x0c, 0x03, size, 0x00, 0); 
-	gp_port_read (port, data, size); 
-	return GP_OK;
-}
-
 int 
 digi_init (GPPort *port, CameraPrivateLibrary *priv)
 {
-	unsigned char c[0x14];
+	char c[0x14];
 	int i;
 	unsigned char *catalog = malloc(0x4000);
 	unsigned char *catalog_tmp;
@@ -62,7 +54,7 @@ digi_init (GPPort *port, CameraPrivateLibrary *priv)
 	gp_port_read (port, c, 0x14); 	
 	digi_reset(port);
 	SQWRITE (port, 0x0c, 0x20, 0x40, NULL, 0);
-	gp_port_read(port, catalog, 0x4000); /* We need 16 bytes for each photo. */
+	gp_port_read(port, (char *)catalog, 0x4000); /* We need 16 bytes for each photo. */
 	digi_reset (port);
 
 	/* The first occurence of a zero denotes end of files entries */
@@ -83,24 +75,24 @@ digi_init (GPPort *port, CameraPrivateLibrary *priv)
 }
 
 
-unsigned char
+int
 digi_get_comp_ratio (CameraPrivateLibrary *priv, int entry)
 {
 	switch (priv->catalog[16*entry]) {
 	case 0x61:
 	case 0x62:
 	case 0x63:
-	case 0x76: return 2;
+	case 0x76: return 1;
 	case 0x41:
 	case 0x42:
 	case 0x43:
 	case 0x52:
 	case 0x53:
 	case 0x56: 
-	case 0x72: return 1;
+	case 0x72: return 0;
 	default:
 		GP_DEBUG ("Your camera has unknown resolution settings.\n");
-			return 0;
+			return GP_ERROR;
 	}
 }
 
@@ -136,7 +128,7 @@ digi_get_picture_width (CameraPrivateLibrary *priv, int entry)
 int
 digi_rewind (GPPort *port, CameraPrivateLibrary *priv)
 {
-	static unsigned char dummy_buf[0x4000];
+	static char dummy_buf[0x4000];
 	
 	
 	GP_DEBUG("REWIND cam's data pointer");
@@ -164,7 +156,8 @@ digi_reset (GPPort *port)
     	return GP_OK;
 }
 
-unsigned char *
+
+int
 digi_read_picture_data (GPPort *port, unsigned char *data, int size, int n )
 {
 
@@ -175,10 +168,55 @@ digi_read_picture_data (GPPort *port, unsigned char *data, int size, int n )
     		SQWRITE (port, 0x0c, 0x30, 0x00, NULL, 0); 
 	}	
 	while ((offset + 0x8000 < size)) {
-		gp_port_read (port, data + offset, 0x8000);
+		gp_port_read (port, (char *)data + offset, 0x8000);
 			offset = offset + 0x8000;
 	}
- 	gp_port_read (port, data + offset, remainder);
+ 	gp_port_read (port, (char *)data + offset, remainder);
 
     	return GP_OK;
 } 
+
+int digi_delete_all     (GPPort *port, CameraPrivateLibrary *priv) 
+{
+	int i;
+	int size;
+	int num_pics;
+	unsigned char *get_size;
+	unsigned char *junk=NULL;
+	get_size=malloc(0x50);
+	if(!get_size) 
+		return GP_ERROR_NO_MEMORY;
+	num_pics = priv->nb_entries;
+	GP_DEBUG("number of entries is %i\n", num_pics);
+	digi_reset (port);
+	digi_reset (port);
+	if(!num_pics) {
+		GP_DEBUG("Camera is already empty!\n");
+		return GP_OK;
+	}	
+	SQWRITE (port, 0x0c, 0x1440, 0x110f, NULL, 0);
+	i=0;
+    	while (i < 19) {
+		gp_port_read(port, (char *)get_size, 0x50);
+		GP_DEBUG("get_size[0x40] = 0x%x\n", get_size[0x40]);	
+		size = get_size[0x40]+(get_size[0x41]*0x100);
+		GP_DEBUG("size = 0x%x\n", size);
+		if(size <= 0xff) {
+			free(get_size);
+			GP_DEBUG("Size not read. Downloading to clear camera not needed.\n");
+			digi_reset(port);
+			return GP_OK;
+		}
+		junk = malloc(size);
+		if(! junk) {
+			GP_DEBUG("allocation of junk space failed\n");
+			return GP_ERROR_NO_MEMORY;	
+		}
+		gp_port_read(port, (char *)junk, size);
+		free(junk); 
+		i ++;
+	}
+	digi_reset (port);
+	return GP_OK;
+}
+
