@@ -36,8 +36,8 @@
 
 #define GP_MODULE "digita"
 
-#include <gphoto2.h>
-#include <gphoto2-port.h>
+#include <gphoto2/gphoto2.h>
+#include <gphoto2/gphoto2-port.h>
 
 #ifdef ENABLE_NLS
 #  include <libintl.h>
@@ -62,7 +62,7 @@
 # define MIN(a, b) ((a) < (b) ? (a) : (b))
 #endif
 
-static struct {
+static const struct {
 	char *model;
 	int usb_vendor;
 	int usb_product;
@@ -74,6 +74,7 @@ static struct {
 	{"HP:PhotoSmart C500",	0x03F0, 0x4102 },
 	{"HP:PhotoSmart 618",	0x03F0, 0x4102 },
 	{"HP:PhotoSmart 912",	0x03F0, 0x4102 },
+	{"HP:PhotoSmart C500 2",0xf003, 0x6002 },
 };
 
 int camera_abilities(CameraAbilitiesList *list)
@@ -240,6 +241,7 @@ static unsigned char *digita_file_get(Camera *camera, const char *folder,
 
 	if (digita_get_file_data(camera->pl, thumbnail, &fn, &tag, data) < 0) {
 		GP_DEBUG( "digita_get_picture: digita_get_file_data failed");
+		free (data);
 		return NULL;
 	}
 
@@ -267,6 +269,7 @@ static unsigned char *digita_file_get(Camera *camera, const char *folder,
 
 		if (digita_get_file_data(camera->pl, thumbnail, &fn, &tag, data + pos) < 0) {
 			GP_DEBUG ("digita_get_file_data failed.");
+			free (data);
 			return NULL;
 		}
 		pos += ntohl(tag.length);
