@@ -25,8 +25,8 @@
 #include <unistd.h>
 #include <string.h>
 
-#include <gphoto2.h>
-#include <gphoto2-port.h>
+#include <gphoto2/gphoto2.h>
+#include <gphoto2/gphoto2-port.h>
 
 #include <bayer.h>
 
@@ -400,8 +400,10 @@ jd11_index_reader(GPPort *port, CameraFilesystem *fs, GPContext *context) {
 	CameraFileInfo	info;
 	
 	ret = gp_file_new(&file);
-	if (ret!=GP_OK)
+	if (ret!=GP_OK) {
+	    free(indexbuf);
 	    return ret;
+	}
 	sprintf(fn,"image%02i.pgm",i);
 	gp_file_set_type (file, GP_FILE_TYPE_PREVIEW);
 	gp_file_set_name(file, fn);
@@ -414,9 +416,16 @@ jd11_index_reader(GPPort *port, CameraFilesystem *fs, GPContext *context) {
 		thumb[47*64-off+(63-x)] = src[off+x];
 	}
 	ret = gp_file_append(file,thumb,sizeof(thumb));
-	if (ret != GP_OK) return ret;
+	if (ret != GP_OK) {
+		gp_file_free (file);
+		return ret;
+	}
 	ret = gp_filesystem_append(fs, "/", fn, context);
-	if (ret != GP_OK) return ret;
+	if (ret != GP_OK) {
+		/* should perhaps remove the entry again */
+		gp_file_free (file);
+		return ret;
+	}
 	ret = gp_filesystem_set_file_noop(fs, "/", file, context);
 	if (ret != GP_OK) return ret;
 
