@@ -12,7 +12,7 @@
 #ifndef __DC3200_H__
 #define __DC3200_H__
 
-#include <gphoto2-camera.h>
+#include <gphoto2/gphoto2-camera.h>
 
 #define CMD_LIST_FILES		0
 #define CMD_GET_PREVIEW		1
@@ -29,7 +29,6 @@ struct _CameraPrivateLibrary {
 	GPContext		*context;	/* for progress updates */
 };
 
-int init(Camera *camera);
 int check_last_use(Camera *camera);
 
 #endif
