@@ -19,7 +19,7 @@
 *                                                                     *
 **********************************************************************/
 
-/* $Id: upload.c,v 1.10 2002/12/15 11:43:26 asl Exp $ */
+/* $Id: upload.c 9891 2007-01-16 05:07:28Z gus_hartmann $ */
 
 #include "config.h"
 
@@ -31,7 +31,7 @@ int dimagev_put_file(dimagev_t* dimagev, CameraFile *file) {
 	unsigned char total_packets= (unsigned char) 0, sent_packets= (unsigned char) 0;
 	int left_to_send=0;
 	const char *data;
-	long int size;
+	unsigned long int size;
 
 	dimagev_packet *p;
 	unsigned char char_buffer, command_buffer[3], *packet_buffer;
