@@ -89,41 +89,46 @@ dtoh64ap (PTPParams *params, unsigned char *a)
 static inline char*
 ptp_unpack_string(PTPParams *params, unsigned char* data, uint16_t offset, uint8_t *len)
 {
-	int i;
-	uint8_t loclen;
+	uint8_t length;
+	uint16_t string[PTP_MAXSTRLEN+1];
+	/* allow for UTF-8: max of 3 bytes per UCS-2 char, plus final null */
+	char loclstr[PTP_MAXSTRLEN*3+1]; 
+	size_t nconv, srclen, destlen;
+	char *src, *dest;
 
-	/* Cannot exceed 255 (PTP_MAXSTRLEN) since it is a single byte, duh ... */
-	loclen = dtoh8a(&data[offset]);
-	/* This len is used to advance the buffer pointer */
-	*len = loclen;
-	if (loclen) {
-		uint16_t string[PTP_MAXSTRLEN+1];
-		char *stringp = (char *) string;
-		char loclstr[PTP_MAXSTRLEN*3+1]; /* UTF-8 encoding is max 3 bytes per UCS2 char. */
-		char *locp = loclstr;
-		size_t nconv;
-		size_t convlen = loclen * 2; /* UCS-2 is 16 bit wide */
-		size_t convmax = PTP_MAXSTRLEN*3;
-		
-		for (i=0;i<loclen;i++) {
-			string[i]=dtoh16a(&data[offset+i*2+1]);
+	length = dtoh8a(&data[offset]);	/* PTP_MAXSTRLEN == 255, 8 bit len */
+	*len = length;
+	if (length == 0)		/* nothing to do? */
+		return(NULL);
+
+	/* copy to string[] to ensure correct alignment for iconv(3) */
+	memcpy(string, &data[offset+1], length * sizeof(string[0]));
+	string[length] = 0x0000U;   /* be paranoid!  add a terminator. */
+	loclstr[0] = '\0';
+    
+	/* convert from camera UCS-2 to our locale */
+	src = (char *)string;
+	srclen = length * sizeof(string[0]);
+	dest = loclstr;
+	destlen = sizeof(loclstr)-1;
+	nconv = iconv(params->cd_ucs2_to_locale, &src, &srclen, 
+			&dest, &destlen);
+	if (nconv == (size_t) -1) { /* do it the hard way */
+		int i;
+		/* try the old way, in case iconv is broken */
+		for (i=0;i<length;i++) {
+			if (dtoh16a(&data[offset+1+2*i])>127)
+				loclstr[i] = '?';
+			else
+				loclstr[i] = dtoh16a(&data[offset+1+2*i]);
 		}
-		/* be paranoid! Add a terminator. :( */
-		string[loclen]=0x0000U;
-		loclstr[0]='\0';
-		/* loclstr=ucs2_to_utf8(string); */
-		/* Do the conversion.  */
-		nconv = iconv (params->cd_ucs2_to_locale, &stringp, &convlen, &locp, &convmax);
-		/* FIXME: handle size errors */
-		loclstr[PTP_MAXSTRLEN*3] = '\0';
-		if (nconv == (size_t) -1)
-			return NULL;
-		return strdup(loclstr);
+		dest = loclstr+length;
 	}
-	return NULL;
+	*dest = '\0';
+	loclstr[sizeof(loclstr)-1] = '\0';   /* be safe? */
+	return(strdup(loclstr));
 }
 
-
 static inline int
 ucs2strlen(uint16_t const * const unicstr)
 {
@@ -138,7 +143,6 @@ ucs2strlen(uint16_t const * const unicstr)
 static inline void
 ptp_pack_string(PTPParams *params, char *string, unsigned char* data, uint16_t offset, uint8_t *len)
 {
-	int i;
 	int packedlen;
 	uint16_t ucs2str[PTP_MAXSTRLEN+1];
 	char *ucs2strp = (char *) ucs2str;
@@ -148,25 +152,28 @@ ptp_pack_string(PTPParams *params, char *string, unsigned char* data, uint16_t o
 	size_t convmax = PTP_MAXSTRLEN * 2; /* Includes the terminator */
 
 	/* Cannot exceed 255 (PTP_MAXSTRLEN) since it is a single byte, duh ... */
-	ucs2str[0] = 0x0000U;
-	memset(ucs2strp, 0, PTP_MAXSTRLEN*2+2);
-	nconv = iconv (params->cd_locale_to_ucs2, &stringp, &convlen, &ucs2strp, &convmax);
+	memset(ucs2strp, 0, sizeof(ucs2str));  /* XXX: necessary? */
+	nconv = iconv(params->cd_locale_to_ucs2, &stringp, &convlen,
+		&ucs2strp, &convmax);
 	if (nconv == (size_t) -1)
 		ucs2str[0] = 0x0000U;
+	/*
+	 * XXX: isn't packedlen just ( (uint16_t *)ucs2strp - ucs2str )?
+	 *      why do we need ucs2strlen()?
+	 */
 	packedlen = ucs2strlen(ucs2str);
 	if (packedlen > PTP_MAXSTRLEN-1) {
 		*len=0;
 		return;
 	}
 	
-	/* number of characters + leading 0 */
+	/* number of characters including terminating 0 (PTP standard confirmed) */
 	htod8a(&data[offset],packedlen+1);
-	for (i=0;i<packedlen && i< PTP_MAXSTRLEN; i++) {
-		htod16a(&data[offset+i*2+1],ucs2str[i]);
-	}
+	memcpy(&data[offset+1], &ucs2str[0], packedlen * sizeof(ucs2str[0]));
+	htod16a(&data[offset+packedlen*2+1], 0x0000);  /* terminate 0 */
 
 	/* The returned length is in number of characters */
-	*len = (uint8_t) packedlen;
+	*len = (uint8_t) packedlen+1;
 }
 
 static inline unsigned char *
@@ -179,11 +186,7 @@ ptp_get_packed_stringcopy(PTPParams *params, char *string, uint32_t *packed_size
 	ptp_pack_string(params, string, (unsigned char*) packed, 0, &len);
 	/* returned length is in characters, then one byte for string length */
 	plen = len*2 + 1;
-	/* Assure proper termination, two zero bytes */
-	packed[plen] = 0x00;
-	packed[plen+1] = 0x00;
-	/* Include terminator */
-	plen += 2;
+	
 	retcopy = malloc(plen);
 	if (!retcopy) {
 		*packed_size = 0;
@@ -297,6 +300,20 @@ ptp_unpack_DI (PTPParams *params, unsigned char* data, PTPDeviceInfo *di, unsign
 		PTP_di_OperationsSupported+totallen,
 		&len);
 }
+
+static void
+ptp_free_DI (PTPDeviceInfo *di) {
+	if (di->SerialNumber) free (di->SerialNumber);
+	if (di->DeviceVersion) free (di->DeviceVersion);
+	if (di->Model) free (di->Model);
+	if (di->Manufacturer) free (di->Manufacturer);
+	if (di->ImageFormats) free (di->ImageFormats);
+	if (di->CaptureFormats) free (di->CaptureFormats);
+	if (di->VendorExtensionDesc) free (di->VendorExtensionDesc);
+	if (di->OperationsSupported) free (di->OperationsSupported);
+	if (di->EventsSupported) free (di->EventsSupported);
+	if (di->DevicePropertiesSupported) free (di->DevicePropertiesSupported);
+}
 	
 /* ObjectHandles array pack/unpack */
 
@@ -305,7 +322,12 @@ ptp_unpack_DI (PTPParams *params, unsigned char* data, PTPDeviceInfo *di, unsign
 static inline void
 ptp_unpack_OH (PTPParams *params, unsigned char* data, PTPObjectHandles *oh, unsigned int len)
 {
-	oh->n = ptp_unpack_uint32_t_array(params, data, PTP_oh, &oh->Handler);
+	if (len) {
+		oh->n = ptp_unpack_uint32_t_array(params, data, PTP_oh, &oh->Handler);
+	} else {
+		oh->n = 0;
+		oh->Handler = NULL;
+	} 
 }
 
 /* StoreIDs array pack/unpack */
@@ -367,19 +389,23 @@ ptp_unpack_SI (PTPParams *params, unsigned char* data, PTPStorageInfo *si, unsig
 #define PTP_oi_filenamelen		52
 #define PTP_oi_Filename			53
 
+/* the max length assuming zero length dates. We have need 3 */
+/* bytes for these. */
+#define PTP_oi_MaxLen PTP_oi_Filename+(PTP_MAXSTRLEN+1)*2+3
+
 static inline uint32_t
 ptp_pack_OI (PTPParams *params, PTPObjectInfo *oi, unsigned char** oidataptr)
 {
 	unsigned char* oidata;
 	uint8_t filenamelen;
 	uint8_t capturedatelen=0;
-	/* let's allocate some memory first; XXX i'm sure it's wrong */
-	oidata=malloc(PTP_oi_Filename+(strlen(oi->Filename)+1)*2+4);
+	/* let's allocate some memory first; correct assuming zero length dates */
+	oidata=malloc(PTP_oi_MaxLen);
 	/* the caller should free it after use! */
 #if 0
 	char *capture_date="20020101T010101"; /* XXX Fake date */
 #endif
-	memset (oidata, 0, (PTP_oi_Filename+(strlen(oi->Filename)+1)*2+4));
+	memset (oidata, 0, PTP_oi_MaxLen);
 	htod32a(&oidata[PTP_oi_StorageID],oi->StorageID);
 	htod16a(&oidata[PTP_oi_ObjectFormat],oi->ObjectFormat);
 	htod16a(&oidata[PTP_oi_ProtectionStatus],oi->ProtectionStatus);
@@ -426,7 +452,7 @@ ptp_pack_OI (PTPParams *params, PTPObjectInfo *oi, unsigned char** oidataptr)
 	/* XXX this function should return dataset length */
 	
 	*oidataptr=oidata;
-	return (PTP_oi_Filename+(filenamelen+1)*2+(capturedatelen+1)*4);
+	return (PTP_oi_Filename+filenamelen*2+(capturedatelen+1)*3);
 }
 
 static inline void
@@ -561,6 +587,28 @@ ptp_unpack_DPV (
 	case PTP_DTC_UINT32:
 		CTVAL(value->u32,dtoh32a);
 		break;
+
+
+
+	case PTP_DTC_UINT64:
+		*offset += 8;
+		/*fprintf(stderr,"unhandled unpack of uint64\n");*/
+		break;
+	case PTP_DTC_INT64:
+		*offset += 8;
+		/*fprintf(stderr,"unhandled unpack of int64\n");*/
+		break;
+	case PTP_DTC_UINT128:
+		*offset += 16;
+		/*fprintf(stderr,"unhandled unpack of uint128n");*/
+		break;
+	case PTP_DTC_INT128:
+		*offset += 16;
+		/*fprintf(stderr,"unhandled unpack of int128n");*/
+		break;
+
+
+
 	case PTP_DTC_AINT8:
 		RARR(value,i8,dtoh8a);
 		break;
@@ -590,6 +638,8 @@ ptp_unpack_DPV (
 			return 0;
 		break;
 	}
+	default:
+		return 0;
 	}
 	return 1;
 }
@@ -834,12 +884,13 @@ ptp_pack_DPV (PTPParams *params, PTPPropertyValue* value, unsigned char** dpvptr
 
 #define MAX_MTP_PROPS 127
 static inline uint32_t
-ptp_pack_OPL (PTPParams *params, MTPPropList *proplist, unsigned char** opldataptr, uint32_t objectid)
+ptp_pack_OPL (PTPParams *params, MTPPropList *proplist, unsigned char** opldataptr)
 {
 	unsigned char* opldata;
 	MTPPropList *propitr;
 	unsigned char *packedprops[MAX_MTP_PROPS];
 	uint32_t packedpropslens[MAX_MTP_PROPS];
+	uint32_t packedobjecthandles[MAX_MTP_PROPS];
 	uint16_t packedpropsids[MAX_MTP_PROPS];
 	uint16_t packedpropstypes[MAX_MTP_PROPS];
 	uint32_t totalsize = 0;
@@ -850,7 +901,8 @@ ptp_pack_OPL (PTPParams *params, MTPPropList *proplist, unsigned char** opldatap
 	totalsize = sizeof(uint32_t); /* 4 bytes to store the number of elements */
 	propitr = proplist;
 	while (propitr != NULL && noitems < MAX_MTP_PROPS) {
-		/* Overhead for each item */
+		/* Object Handle */
+		packedobjecthandles[noitems]=propitr->ObjectHandle;
 		totalsize += sizeof(uint32_t); /* Object ID */
 		/* Metadata type */
 		packedpropsids[noitems]=propitr->property;
@@ -874,7 +926,7 @@ ptp_pack_OPL (PTPParams *params, MTPPropList *proplist, unsigned char** opldatap
 	/* Copy into a nice packed list */
 	for (i = 0; i < noitems; i++) {
 		/* Object ID */
-		htod32a(&opldata[bufp],objectid);
+		htod32a(&opldata[bufp],packedobjecthandles[i]);
 		bufp += sizeof(uint32_t);
 		htod16a(&opldata[bufp],packedpropsids[i]);
 		bufp += sizeof(uint16_t);
@@ -904,15 +956,15 @@ ptp_unpack_OPL (PTPParams *params, unsigned char* data, MTPPropList **proplist,
 	*proplist = malloc(sizeof(MTPPropList));
 	prop = *proplist;
 	for (i = 0; i < prop_count; i++) {
-		/* we ignore the object handle */
+		prop->ObjectHandle = dtoh32a(data);
 		data += sizeof(uint32_t);
 		len -= sizeof(uint32_t);
 
-		prop->property = dtoh32a(data);
+		prop->property = dtoh16a(data);
 		data += sizeof(uint16_t);
 		len -= sizeof(uint16_t);
 
-		prop->datatype = dtoh32a(data);
+		prop->datatype = dtoh16a(data);
 		data += sizeof(uint16_t);
 		len -= sizeof(uint16_t);
 
@@ -993,6 +1045,195 @@ ptp_unpack_Canon_FE (PTPParams *params, unsigned char* data, PTPCANONFolderEntry
 		fe->Filename[i]=(char)dtoh8a(&data[PTP_cfe_Filename+i]);
 }
 
+/*
+    PTP EOS Changes Entry unpack
+*/
+#define PTP_ece_Size		0
+#define PTP_ece_Type		4
+
+#define PTP_ece_Prop_Subtype	8	/* only for properties */
+#define PTP_ece_Prop_Val_Data	0xc	/* only for properties */
+#define PTP_ece_Prop_Desc_Type	0xc	/* only for property descs */
+#define PTP_ece_Prop_Desc_Count	0x10	/* only for property descs */
+#define PTP_ece_Prop_Desc_Data	0x14	/* only for property descs */
+
+#define PTP_ece_OI_ObjectID	8	/* only for objectinfos */
+#define PTP_ece_OI_OFC		0x0c	/* only for objectinfos */
+#define PTP_ece_OI_Size		0x14	/* only for objectinfos */
+#define PTP_ece_OI_Name		0x1c	/* only for objectinfos */
+
+static inline int
+ptp_unpack_CANON_changes (PTPParams *params, unsigned char* data, int datasize, PTPCanon_changes_entry **ce)
+{
+	int	i = 0, entries = 0;
+	unsigned char	*curdata = data;
+
+	if (data==NULL)
+		return 0;
+	while (curdata - data < datasize) {
+		uint32_t	size = dtoh32a(&curdata[PTP_ece_Size]);
+		uint32_t	type = dtoh32a(&curdata[PTP_ece_Type]);
+
+		curdata += size;
+		if ((size == 8) && (type == 0))
+			break;
+		entries++;
+	}
+	*ce = malloc (sizeof(PTPCanon_changes_entry)*entries);
+	if (!*ce) return 0;
+
+	curdata = data;
+	while (curdata - data < datasize) {
+		uint32_t	size = dtoh32a(&curdata[PTP_ece_Size]);
+		uint32_t	type = dtoh32a(&curdata[PTP_ece_Type]);
+
+		switch (type) {
+		case  0xc186: {	/* objectinfo from capture */
+			(*ce)[i].type = PTP_CANON_EOS_CHANGES_TYPE_OBJECTINFO;
+			(*ce)[i].u.object.oid    		= dtoh32a(&curdata[PTP_ece_OI_ObjectID]);
+			(*ce)[i].u.object.oi.ObjectFormat 	= dtoh16a(&curdata[PTP_ece_OI_OFC]);
+			(*ce)[i].u.object.oi.ObjectCompressedSize = dtoh32a(&curdata[PTP_ece_OI_Size]);
+			(*ce)[i].u.object.oi.Filename 		= strdup(((char*)&curdata[PTP_ece_OI_Name]));
+			break;
+		}
+		case  0xc18a: {	/* property desc */
+			uint32_t	proptype = dtoh32a(&curdata[PTP_ece_Prop_Subtype]);
+			uint32_t	propxtype = dtoh32a(&curdata[PTP_ece_Prop_Desc_Type]);
+			uint32_t	propxcnt = dtoh32a(&curdata[PTP_ece_Prop_Desc_Count]);
+			unsigned char	*data = &curdata[PTP_ece_Prop_Desc_Data];
+			int		j;
+			PTPDevicePropDesc	*dpd;
+
+			/*fprintf (stderr, "Adding EOS property %04x desc record, datasize is %d\n", proptype, size-PTP_ece_Prop_Desc_Data);*/
+			for (j=0;j<params->nrofcanon_props;j++)
+				if (params->canon_props[j].proptype == proptype)
+					break;
+			if (j==params->nrofcanon_props) {
+				/*fprintf (stderr, "should have received default value for %x first!\n", proptype);*/
+				break;
+			}
+			dpd = &params->canon_props[j].dpd;
+			if (propxtype != 3) {
+				/*fprintf (stderr, "propxtype is %x for %x, unhandled.\n", propxtype, proptype);*/
+				break;
+			}
+			if (propxcnt) {
+				dpd->FormFlag = PTP_DPFF_Enumeration;
+				dpd->FORM.Enum.NumberOfValues = propxcnt;
+				dpd->FORM.Enum.SupportedValue = malloc (sizeof (PTPPropertyValue)*propxcnt);
+				for (j=0;j<propxcnt;j++) {
+					switch (dpd->DataType) {
+					case PTP_DTC_UINT16:
+						dpd->FORM.Enum.SupportedValue[j].u16	= dtoh16a(data);
+						/*fprintf (stderr,"suppvalue[%d] of %x is %x\n", j, proptype, dtoh16a(data));*/
+						break;
+					case PTP_DTC_UINT8:
+						dpd->FORM.Enum.SupportedValue[j].u8	= dtoh8a(data);
+						/*fprintf (stderr,"suppvalue[%d] of %x is %x\n", j, proptype, dtoh8a(data));*/
+						break;
+					default:
+						/*fprintf(stderr,"data type 0x%04x of %x unhandled, fill in (val=%x).\n", dpd->DataType, proptype, dtoh32a(data));*/
+						break;
+					}
+					data += 4; /* might only be for propxtype 3 */
+				}
+			}
+			break;
+		}
+		case  0xc189:	/* property value */
+			if (size >= 0xc) {	/* property info */
+				int j;
+				uint32_t	proptype = dtoh32a(&curdata[PTP_ece_Prop_Subtype]);
+				unsigned char	*data = &curdata[PTP_ece_Prop_Val_Data];
+				PTPDevicePropDesc	*dpd;
+
+				for (j=0;j<params->nrofcanon_props;j++)
+					if (params->canon_props[j].proptype == proptype)
+						break;
+				if (j<params->nrofcanon_props) {
+					if (	(params->canon_props[j].size != size) ||
+						(memcmp(params->canon_props[j].data,data,size-PTP_ece_Prop_Val_Data))) {
+						params->canon_props[j].data = realloc(params->canon_props[j].data,size-PTP_ece_Prop_Val_Data);
+						memcpy (params->canon_props[j].data,data,size-PTP_ece_Prop_Val_Data);
+					}
+				} else {
+					if (j)
+						params->canon_props = realloc(params->canon_props, sizeof(params->canon_props[0])*(j+1));
+					else
+						params->canon_props = malloc(sizeof(params->canon_props[0]));
+					params->canon_props[j].type = type;
+					params->canon_props[j].proptype = proptype;
+					params->canon_props[j].size = size;
+					params->canon_props[j].data = malloc(size-PTP_ece_Prop_Val_Data);
+					memcpy(params->canon_props[j].data, data, size-PTP_ece_Prop_Val_Data);
+					memset (&params->canon_props[j].dpd,0,sizeof(params->canon_props[j].dpd));
+					params->canon_props[j].dpd.GetSet = 1;
+					params->canon_props[j].dpd.FormFlag = PTP_DPFF_None;
+					params->nrofcanon_props = j+1;
+				}
+				dpd = &params->canon_props[j].dpd;
+				switch (proptype) {
+				case PTP_DPC_CANON_EOS_CameraTime:
+					dpd->DataType = PTP_DTC_UINT32;
+					break;
+				case PTP_DPC_CANON_EOS_Aperture:
+				case PTP_DPC_CANON_EOS_ShutterSpeed:
+				case PTP_DPC_CANON_EOS_ISOSpeed:
+					dpd->DataType = PTP_DTC_UINT16;
+					break;
+				case PTP_DPC_CANON_EOS_PictureStyle:
+				case PTP_DPC_CANON_EOS_WhiteBalance:
+				case PTP_DPC_CANON_EOS_MeteringMode:
+				case PTP_DPC_CANON_EOS_ExpCompensation:
+					dpd->DataType = PTP_DTC_UINT8;
+					break;
+				case PTP_DPC_CANON_EOS_Owner:
+					dpd->DataType = PTP_DTC_STR;
+					break;
+				default:
+					/*fprintf (stderr, "Unknown EOS property %04x, datasize is %d\n", proptype, size-PTP_ece_Prop_Val_Data);*/
+					break;
+				}
+				switch (dpd->DataType) {
+				case PTP_DTC_UINT32:
+					dpd->FactoryDefaultValue.u32	= dtoh32a(data);
+					dpd->CurrentValue.u32		= dtoh32a(data);
+					/*fprintf (stderr,"currentvalue of %x is %x\n", proptype, dpd->CurrentValue.u16);*/
+					break;
+				case PTP_DTC_UINT16:
+					dpd->FactoryDefaultValue.u16	= dtoh16a(data);
+					dpd->CurrentValue.u16		= dtoh16a(data);
+					/*fprintf (stderr,"currentvalue of %x is %x\n", proptype, dpd->CurrentValue.u16);*/
+					break;
+				case PTP_DTC_UINT8:
+					dpd->FactoryDefaultValue.u8	= dtoh8a(data);
+					dpd->CurrentValue.u8		= dtoh8a(data);
+					/*fprintf (stderr,"currentvalue of %x is %x\n", proptype, dpd->CurrentValue.u8);*/
+					break;
+				case PTP_DTC_STR: {
+					uint8_t len = 0;
+					dpd->FactoryDefaultValue.str	= ptp_unpack_string(params, data, 0, &len);
+					dpd->CurrentValue.str		= ptp_unpack_string(params, data, 0, &len);
+					break;
+				}
+				default:
+					/* debug is printed in switch above this one */
+					break;
+				}
+				break;
+		}
+		default:
+			/* fprintf (stderr, "unknown EOS property type %04x\n", type); */
+			(*ce)[i].type = PTP_CANON_EOS_CHANGES_TYPE_UNKNOWN;
+			break;
+		}
+		curdata += size;
+		i++;
+	}
+
+	return entries;
+}
+
 /*
     PTP USB Event container unpack for Nikon events.
 */
