@@ -26,7 +26,7 @@
 #include <string.h>
 #include <time.h>
 
-#include <gphoto2-port-log.h>
+#include <gphoto2/gphoto2-port-log.h>
 
 #include "sierra.h"
 #include "sierra-usbwrap.h"
@@ -98,7 +98,7 @@ int sierra_change_folder (Camera *camera, const char *folder, GPContext *context
 
 	/* We will later need a path ending with '/'. */
 	memset (target, 0, sizeof (target));
-	if (folder && folder[0])
+	if (folder[0])
 		strncpy (target, folder, sizeof (target) - 1);
 	if (target[strlen (target) - 1] != '/')
 		strcat (target, "/");
@@ -629,7 +629,7 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 
 		/* 
 		 * Read until the end of the packet is reached
-		 * or an error occured.
+		 * or an error occurred.
 		 */
 		while (br < length) {
 			result = gp_port_read (camera->port, packet + br,
