@@ -71,10 +71,14 @@ struct _CameraAbilitiesList {
 	CameraAbilities *abilities;
 };
 
+/** \internal */
 static int gp_abilities_list_lookup_id (CameraAbilitiesList *, const char *);
+/** \internal */
 static int gp_abilities_list_sort      (CameraAbilitiesList *);
 
 /**
+ * \brief Set the current character codeset libgphoto2 is operating in.
+ *
  * Set the codeset for all messages returned by libgphoto2.
  * \param codeset New codeset for the messages. For instance "utf-8".
  * \return old codeset as returned from bind_textdomain_codeset().
@@ -90,7 +94,9 @@ gp_message_codeset (const char *codeset)
 }
 
 /**
- * Allocate the memory for a new abilities list.
+ * \brief Allocate the memory for a new abilities list.
+ *
+ * Function to allocate the memory for a new abilities list.
  * \param list CameraAbilitiesList object to initialize
  * \return gphoto2 error code
  *
@@ -162,7 +168,7 @@ gp_abilities_list_load_dir (CameraAbilitiesList *list, const char *dir,
 	CameraLibraryIdFunc id;
 	CameraLibraryAbilitiesFunc ab;
 	CameraText text;
-	int x, old_count, new_count;
+	int ret, x, old_count, new_count;
 	unsigned int i, p;
 	const char *filename;
 	CameraList *flist;
@@ -174,7 +180,11 @@ gp_abilities_list_load_dir (CameraAbilitiesList *list, const char *dir,
 	gp_log (GP_LOG_DEBUG, "gphoto2-abilities-list",
 		"Using ltdl to load camera libraries from '%s'...", dir);
 	CHECK_RESULT (gp_list_new (&flist));
-	CHECK_RESULT (gp_list_reset (flist));
+	ret = gp_list_reset (flist);
+	if (ret < GP_OK) {
+		gp_list_free (flist);
+		return ret;
+	}
 	if (1) { /* a new block in which we can define a temporary variable */
 		int ret;
 		foreach_data_t foreach_data = { flist, GP_OK };
@@ -192,14 +202,22 @@ gp_abilities_list_load_dir (CameraAbilitiesList *list, const char *dir,
 			return (foreach_data.result!=GP_OK)?foreach_data.result:GP_ERROR;
 		}
 	}
-	CHECK_RESULT (count = gp_list_count (flist));
+	count = gp_list_count (flist);
+	if (count < GP_OK) {
+		gp_list_free (flist);
+		return ret;
+	}
 	gp_log (GP_LOG_DEBUG, "gp-abilities-list", "Found %i "
 		"camera drivers.", count);
 	lt_dlinit ();
 	p = gp_context_progress_start (context, count,
 		_("Loading camera drivers from '%s'..."), dir);
 	for (i = 0; i < count; i++) {
-		CHECK_RESULT (gp_list_get_name (flist, i, &filename));
+		ret = gp_list_get_name (flist, i, &filename);
+		if (ret < GP_OK) {
+			gp_list_free (flist);
+			return ret;
+		}
 		lh = lt_dlopenext (filename);
 		if (!lh) {
 			gp_log (GP_LOG_DEBUG, "gphoto2-abilities-list",
@@ -385,13 +403,14 @@ gp_abilities_list_detect_usb (CameraAbilitiesList *list,
 
 /**
  * \param list a CameraAbilitiesList
- * \param info_list a GPPortInfoList
- * \param l a CameraList
- * \return a gphoto2 error code
+ * \param info_list the GPPortInfoList of ports to use for detection
+ * \param l a #CameraList that contains the autodetected cameras after the call
+ * \param context a #GPContext
  *
  * Tries to detect any camera connected to the computer using the supplied
  * list of supported cameras and the supplied info_list of ports.
  *
+ * \return a gphoto2 error code
  */
 int
 gp_abilities_list_detect (CameraAbilitiesList *list,
@@ -546,8 +565,8 @@ gp_abilities_list_reset (CameraAbilitiesList *list)
 
 /**
  * \brief Count the entries in the supplied list.
- * \param a CameraAbilitiesList
- * \return The number of entries or a gphoto2 error code
+ * \param list a #CameraAbilitiesList
+ * \returns The number of entries or a gphoto2 error code
  */
 int
 gp_abilities_list_count (CameraAbilitiesList *list)
@@ -624,12 +643,13 @@ gp_abilities_list_lookup_model (CameraAbilitiesList *list, const char *model)
 
 /**
  * \brief Retrieve the camera abilities of entry with supplied index number.
+ *
  * \param list a CameraAbilitiesList
  * \param index index
- * \param abilities CameraAbilities
+ * \param abilities pointer to CameraAbilities for returned data.
  * \return a gphoto2 error code
  *
- * Retrieves the camera @abilities of entry with supplied
+ * Retrieves the camera abilities of entry with supplied
  * index number. Typically, you would call gp_camera_set_abilities()
  * afterwards in order to prepare the initialization of a camera.
  */
