@@ -1,17 +1,20 @@
-/* gphoto2-setting.c
+/** \file
  *
- * Copyright  2000 Scott Fritzinger
+ * \author Copyright 2000 Scott Fritzinger
  *
+ * \par License
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
+ * \par
  * This library is distributed in the hope that it will be useful, 
  * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details. 
  *
+ * \par
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
@@ -19,19 +22,22 @@
  */
 
 #include "config.h"
-#include "gphoto2-setting.h"
+#include <gphoto2/gphoto2-setting.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
-#include "gphoto2-result.h"
-#include "gphoto2-port-log.h"
-#include "gphoto2-port-portability.h"
+#include <gphoto2/gphoto2-result.h>
+#include <gphoto2/gphoto2-port-log.h>
+#include <gphoto2/gphoto2-port-portability.h>
 
 #define GP_MODULE "setting"
 
+/**
+ * Internal struct to store settings.
+ */
 typedef struct {
 	/* key = value settings */
 	char id[256];
@@ -40,8 +46,8 @@ typedef struct {
 } Setting;
 
 /* Currently loaded settings */
-int             glob_setting_count = 0;
-Setting         glob_setting[512];
+static int             glob_setting_count = 0;
+static Setting         glob_setting[512];
 
 static int save_settings (void);
 
@@ -50,7 +56,18 @@ static int save_settings (void);
 
 static int load_settings (void);
 
-int gp_setting_get (char *id, char *key, char *value)
+/**
+ * \brief Retrieve a specific gphoto setting.
+ * \param id the frontend id of the caller
+ * \param key the key the frontend queries
+ * \param value retrieved value
+ * \return GPhoto error code
+ *
+ * This function retrieves the setting key for a specific frontend
+ * id and copies the value into the passed value pointer.
+ */
+int
+gp_setting_get (char *id, char *key, char *value)
 {
         int x;
 
@@ -70,6 +87,17 @@ int gp_setting_get (char *id, char *key, char *value)
         return(GP_ERROR);
 }
 
+/**
+ * \brief Set a specific gphoto setting.
+ *
+ * \param id the frontend id of the caller
+ * \param key the key the frontend queries
+ * \param value retrieved value
+ * \return GPhoto error code
+ *
+ * This function sets the setting key for a specific frontend
+ * id to the value.
+ */
 int
 gp_setting_set (char *id, char *key, char *value)
 {
@@ -144,19 +172,19 @@ load_settings (void)
 	/* Make sure the directories are created */
 	GP_DEBUG ("Creating $HOME/.gphoto");
 #ifdef WIN32
-	GetWindowsDirectory (buf, 1024);
+	GetWindowsDirectory (buf, sizeof(buf));
 	strcat (buf, "\\gphoto");
 #else
-	sprintf (buf, "%s/.gphoto", getenv ("HOME"));
+	snprintf (buf, sizeof(buf), "%s/.gphoto", getenv ("HOME"));
 #endif
 	(void)gp_system_mkdir (buf);
 
 	glob_setting_count = 0;
 #ifdef WIN32
-	GetWindowsDirectory(buf, 1024);
+	GetWindowsDirectory(buf, sizeof(buf));
 	strcat(buf, "\\gphoto\\settings");
 #else
-	sprintf(buf, "%s/.gphoto/settings", getenv("HOME"));
+	snprintf(buf, sizeof(buf), "%s/.gphoto/settings", getenv("HOME"));
 #endif
 
 	if (verify_settings(buf) != GP_OK)
@@ -186,7 +214,7 @@ load_settings (void)
 			strcpy(glob_setting[glob_setting_count++].value, "");
 		}
 	}
-
+	fclose (f);
 	return (GP_OK);
 }
 
@@ -198,7 +226,7 @@ save_settings (void)
 	char buf[1024];
 	int x=0;
 
-	sprintf (buf, "%s/.gphoto/settings", getenv ("HOME"));
+	snprintf (buf, sizeof(buf), "%s/.gphoto/settings", getenv ("HOME"));
 
 	gp_log (GP_LOG_DEBUG, "gphoto2-setting",
 		"Saving %i setting(s) to file \"%s\"",
