@@ -28,7 +28,7 @@
  */
 
 #include "config.h"
-#include <gphoto2-port-library.h>
+#include <gphoto2/gphoto2-port-library.h>
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -73,9 +73,9 @@
 #  include <lockdev.h>
 #endif
 
-#include <gphoto2-port-result.h>
-#include <gphoto2-port-log.h>
-#include <gphoto2-port.h>
+#include <gphoto2/gphoto2-port-result.h>
+#include <gphoto2/gphoto2-port-log.h>
+#include <gphoto2/gphoto2-port.h>
 
 #ifdef ENABLE_NLS
 #  include <libintl.h>
@@ -366,7 +366,7 @@ gp_port_library_list (GPPortInfoList *list)
 		gp_port_serial_unlock (NULL, path);
 		info.type = GP_PORT_SERIAL;
 		strncpy (info.path, "serial:", sizeof (info.path));
-		strncat (info.path, path, sizeof (info.path));
+		strncat (info.path, path, sizeof (info.path) - strlen (info.path) - 1);
 		snprintf (info.name, sizeof (info.name),
 			  _("Serial Port %i"), x);
 		CHECK (gp_port_info_list_append (list, info));
@@ -444,7 +444,7 @@ gp_port_serial_open (GPPort *dev)
 	}
 	dev->pl->fd = -1;
 
-#if defined(__FreeBSD__) || defined(__APPLE__)
+#if defined(__FreeBSD__) || defined(__APPLE__) || defined(__DragonFly__)
         dev->pl->fd = open (port, O_RDWR | O_NOCTTY | O_NONBLOCK);
 #elif OS2
         fd = open (port, O_RDWR | O_BINARY);
@@ -486,6 +486,7 @@ gp_port_serial_close (GPPort *dev)
 
 	/* Unlock the port */
 	path = strchr (dev->settings.serial.port, ':');
+	if (!path) return GP_ERROR_BAD_PARAMETERS;
 	path++;
 	CHECK (gp_port_serial_unlock (dev, path));
 
